@@ -3,22 +3,18 @@
  * SPDX-License-Identifier: Apache-2.0 */
 
 #include "device/multi/device.h"
-
-#include <sstream>
-#include <stdlib.h>
-
-#include "bvh/multi.h"
-
 #include "device/device.h"
 #include "device/queue.h"
 
+#include <cstdlib>
+#include <functional>
+
+#include "bvh/multi.h"
+
 #include "scene/geometry.h"
 
-#include "util/foreach.h"
 #include "util/list.h"
-#include "util/log.h"
 #include "util/map.h"
-#include "util/time.h"
 
 CCL_NAMESPACE_BEGIN
 
@@ -26,19 +22,21 @@ class MultiDevice : public Device {
  public:
   struct SubDevice {
     Stats stats;
-    Device *device;
+    unique_ptr<Device> device;
     map<device_ptr, device_ptr> ptr_map;
     int peer_island_index = -1;
   };
 
   list<SubDevice> devices;
-  device_ptr unique_key;
+  device_ptr unique_key = 1;
   vector<vector<SubDevice *>> peer_islands;
 
-  MultiDevice(const DeviceInfo &info, Stats &stats, Profiler &profiler)
-      : Device(info, stats, profiler), unique_key(1)
+  MultiDevice(const DeviceInfo &info_, Stats &stats, Profiler &profiler, bool headless)
+      : Device(info_, stats, profiler, headless)
   {
-    foreach (const DeviceInfo &subinfo, info.multi_devices) {
+    verify_hardware_raytracing();
+
+    for (const DeviceInfo &subinfo : this->info.multi_devices) {
       /* Always add CPU devices at the back since GPU devices can change
        * host memory pointers, which CPU uses as device pointer. */
       SubDevice *sub;
@@ -53,11 +51,11 @@ class MultiDevice : public Device {
 
       /* The pointer to 'sub->stats' will stay valid even after new devices
        * are added, since 'devices' is a linked list. */
-      sub->device = Device::create(subinfo, sub->stats, profiler);
+      sub->device = Device::create(subinfo, sub->stats, profiler, headless);
     }
 
     /* Build a list of peer islands for the available render devices */
-    foreach (SubDevice &sub, devices) {
+    for (SubDevice &sub : devices) {
       /* First ensure that every device is in at least once peer island */
       if (sub.peer_island_index < 0) {
         peer_islands.emplace_back();
@@ -70,10 +68,10 @@ class MultiDevice : public Device {
       }
 
       /* Second check peer access between devices and fill up the islands accordingly */
-      foreach (SubDevice &peer_sub, devices) {
+      for (SubDevice &peer_sub : devices) {
         if (peer_sub.peer_island_index < 0 &&
             peer_sub.device->info.type == sub.device->info.type &&
-            peer_sub.device->check_peer_access(sub.device))
+            peer_sub.device->check_peer_access(sub.device.get()))
         {
           peer_sub.peer_island_index = sub.peer_island_index;
           peer_islands[sub.peer_island_index].push_back(&peer_sub);
@@ -82,27 +80,50 @@ class MultiDevice : public Device {
     }
   }
 
-  ~MultiDevice()
+  void verify_hardware_raytracing()
   {
-    foreach (SubDevice &sub, devices)
-      delete sub.device;
+    /* Determine if we can use hardware ray-tracing. It is only supported if all selected
+     * GPU devices support it. Both the backends and scene update code do not support mixed
+     * BVH2 and hardware raytracing. The CPU device will ignore this setting. */
+    bool have_disabled_hardware_rt = false;
+    bool have_enabled_hardware_rt = false;
+
+    for (const DeviceInfo &subinfo : info.multi_devices) {
+      if (subinfo.type != DEVICE_CPU) {
+        if (subinfo.use_hardware_raytracing) {
+          have_enabled_hardware_rt = true;
+        }
+        else {
+          have_disabled_hardware_rt = true;
+        }
+      }
+    }
+
+    info.use_hardware_raytracing = have_enabled_hardware_rt && !have_disabled_hardware_rt;
+
+    for (DeviceInfo &subinfo : info.multi_devices) {
+      if (subinfo.type != DEVICE_CPU) {
+        subinfo.use_hardware_raytracing = info.use_hardware_raytracing;
+      }
+    }
   }
 
   const string &error_message() override
   {
     error_msg.clear();
 
-    foreach (SubDevice &sub, devices)
+    for (SubDevice &sub : devices) {
       error_msg += sub.device->error_message();
+    }
 
     return error_msg;
   }
 
-  virtual BVHLayoutMask get_bvh_layout_mask(uint kernel_features) const override
+  BVHLayoutMask get_bvh_layout_mask(const uint kernel_features) const override
   {
     BVHLayoutMask bvh_layout_mask = BVH_LAYOUT_ALL;
     BVHLayoutMask bvh_layout_mask_all = BVH_LAYOUT_NONE;
-    foreach (const SubDevice &sub_device, devices) {
+    for (const SubDevice &sub_device : devices) {
       BVHLayoutMask device_bvh_layout_mask = sub_device.device->get_bvh_layout_mask(
           kernel_features);
       bvh_layout_mask &= device_bvh_layout_mask;
@@ -152,20 +173,22 @@ class MultiDevice : public Device {
 
   bool load_kernels(const uint kernel_features) override
   {
-    foreach (SubDevice &sub, devices)
+    for (SubDevice &sub : devices) {
       if (!sub.device->load_kernels(kernel_features)) {
         return false;
       }
+    }
 
     return true;
   }
 
   bool load_osl_kernels() override
   {
-    foreach (SubDevice &sub, devices)
+    for (SubDevice &sub : devices) {
       if (!sub.device->load_osl_kernels()) {
         return false;
       }
+    }
 
     return true;
   }
@@ -190,18 +213,22 @@ class MultiDevice : public Device {
     BVHMulti *const bvh_multi = static_cast<BVHMulti *>(bvh);
     bvh_multi->sub_bvhs.resize(devices.size());
 
-    vector<BVHMulti *> geom_bvhs;
+    /* Temporarily move ownership of BVH on geometry to this vector, to swap
+     * it for each sub device. Need to find a better way to handle this. */
+    vector<unique_ptr<BVH>> geom_bvhs;
     geom_bvhs.reserve(bvh->geometry.size());
-    foreach (Geometry *geom, bvh->geometry) {
-      geom_bvhs.push_back(static_cast<BVHMulti *>(geom->bvh));
+    for (Geometry *geom : bvh->geometry) {
+      geom_bvhs.push_back(std::move(geom->bvh));
     }
 
     /* Broadcast acceleration structure build to all render devices */
     size_t i = 0;
-    foreach (SubDevice &sub, devices) {
+    for (SubDevice &sub : devices) {
       /* Change geometry BVH pointers to the sub BVH */
       for (size_t k = 0; k < bvh->geometry.size(); ++k) {
-        bvh->geometry[k]->bvh = geom_bvhs[k]->sub_bvhs[i];
+        bvh->geometry[k]->bvh.release();  // NOLINT: was not actually the owner
+        bvh->geometry[k]->bvh.reset(
+            static_cast<BVHMulti *>(geom_bvhs[k].get())->sub_bvhs[i].get());
       }
 
       if (!bvh_multi->sub_bvhs[i]) {
@@ -243,34 +270,36 @@ class MultiDevice : public Device {
           continue;
         }
 
-        bvh_multi->sub_bvhs[i] = BVH::create(params, bvh->geometry, bvh->objects, sub.device);
+        bvh_multi->sub_bvhs[i] = BVH::create(
+            params, bvh->geometry, bvh->objects, sub.device.get());
       }
 
-      sub.device->build_bvh(bvh_multi->sub_bvhs[i], progress, refit);
+      sub.device->build_bvh(bvh_multi->sub_bvhs[i].get(), progress, refit);
       i++;
     }
 
-    /* Change geometry BVH pointers back to the multi BVH. */
+    /* Change BVH ownership back to Geometry. */
     for (size_t k = 0; k < bvh->geometry.size(); ++k) {
-      bvh->geometry[k]->bvh = geom_bvhs[k];
+      bvh->geometry[k]->bvh.release();  // NOLINT: was not actually the owner
+      bvh->geometry[k]->bvh = std::move(geom_bvhs[k]);
     }
   }
 
-  virtual void *get_cpu_osl_memory() override
+  OSLGlobals *get_cpu_osl_memory() override
   {
     /* Always return the OSL memory of the CPU device (this works since the constructor above
      * guarantees that CPU devices are always added to the back). */
     if (devices.size() > 1 && devices.back().device->info.type != DEVICE_CPU) {
-      return NULL;
+      return nullptr;
     }
     return devices.back().device->get_cpu_osl_memory();
   }
 
   bool is_resident(device_ptr key, Device *sub_device) override
   {
-    foreach (SubDevice &sub, devices) {
-      if (sub.device == sub_device) {
-        return find_matching_mem_device(key, sub)->device == sub_device;
+    for (SubDevice &sub : devices) {
+      if (sub.device.get() == sub_device) {
+        return find_matching_mem_device(key, sub)->device.get() == sub_device;
       }
     }
     return false;
@@ -283,7 +312,7 @@ class MultiDevice : public Device {
     /* Get the memory owner of this key (first try current device, then peer devices) */
     SubDevice *owner_sub = &sub;
     if (owner_sub->ptr_map.find(key) == owner_sub->ptr_map.end()) {
-      foreach (SubDevice *island_sub, peer_islands[sub.peer_island_index]) {
+      for (SubDevice *island_sub : peer_islands[sub.peer_island_index]) {
         if (island_sub != owner_sub && island_sub->ptr_map.find(key) != island_sub->ptr_map.end())
         {
           owner_sub = island_sub;
@@ -299,7 +328,7 @@ class MultiDevice : public Device {
 
     /* Get the memory owner of this key or the device with the lowest memory usage when new */
     SubDevice *owner_sub = island.front();
-    foreach (SubDevice *island_sub, island) {
+    for (SubDevice *island_sub : island) {
       if (key ? (island_sub->ptr_map.find(key) != island_sub->ptr_map.end()) :
                 (island_sub->device->stats.mem_used < owner_sub->device->stats.mem_used))
       {
@@ -309,20 +338,43 @@ class MultiDevice : public Device {
     return owner_sub;
   }
 
-  inline device_ptr find_matching_mem(device_ptr key, SubDevice &sub)
+  device_ptr find_matching_mem(device_ptr key, SubDevice &sub)
   {
     return find_matching_mem_device(key, sub)->ptr_map[key];
   }
 
+  void *host_alloc(const MemoryType type, const size_t size) override
+  {
+    for (SubDevice &sub : devices) {
+      if (sub.device->info.type != DEVICE_CPU) {
+        return sub.device->host_alloc(type, size);
+      }
+    }
+
+    return Device::host_alloc(type, size);
+  }
+
+  void host_free(const MemoryType type, void *host_pointer, const size_t size) override
+  {
+    for (SubDevice &sub : devices) {
+      if (sub.device->info.type != DEVICE_CPU) {
+        sub.device->host_free(type, host_pointer, size);
+        return;
+      }
+    }
+
+    Device::host_free(type, host_pointer, size);
+  }
+
   void mem_alloc(device_memory &mem) override
   {
     device_ptr key = unique_key++;
 
     assert(mem.type == MEM_READ_ONLY || mem.type == MEM_READ_WRITE || mem.type == MEM_DEVICE_ONLY);
     /* The remaining memory types can be distributed across devices */
-    foreach (const vector<SubDevice *> &island, peer_islands) {
+    for (const vector<SubDevice *> &island : peer_islands) {
       SubDevice *owner_sub = find_suitable_mem_device(key, island);
-      mem.device = owner_sub->device;
+      mem.device = owner_sub->device.get();
       mem.device_pointer = 0;
       mem.device_size = 0;
 
@@ -341,10 +393,9 @@ class MultiDevice : public Device {
     device_ptr key = (existing_key) ? existing_key : unique_key++;
     size_t existing_size = mem.device_size;
 
-    /* The tile buffers are allocated on each device (see below), so copy to all of them */
-    foreach (const vector<SubDevice *> &island, peer_islands) {
+    for (const vector<SubDevice *> &island : peer_islands) {
       SubDevice *owner_sub = find_suitable_mem_device(existing_key, island);
-      mem.device = owner_sub->device;
+      mem.device = owner_sub->device.get();
       mem.device_pointer = (existing_key) ? owner_sub->ptr_map[existing_key] : 0;
       mem.device_size = existing_size;
 
@@ -353,7 +404,7 @@ class MultiDevice : public Device {
 
       if (mem.type == MEM_GLOBAL || mem.type == MEM_TEXTURE) {
         /* Need to create texture objects and update pointer in kernel globals on all devices */
-        foreach (SubDevice *island_sub, island) {
+        for (SubDevice *island_sub : island) {
           if (island_sub != owner_sub) {
             island_sub->device->mem_copy_to(mem);
           }
@@ -366,17 +417,69 @@ class MultiDevice : public Device {
     stats.mem_alloc(mem.device_size - existing_size);
   }
 
-  void mem_copy_from(device_memory &mem, size_t y, size_t w, size_t h, size_t elem) override
+  void mem_move_to_host(device_memory &mem) override
+  {
+    assert(mem.type == MEM_GLOBAL || mem.type == MEM_TEXTURE);
+
+    device_ptr existing_key = mem.device_pointer;
+    device_ptr key = (existing_key) ? existing_key : unique_key++;
+    size_t existing_size = mem.device_size;
+
+    for (const vector<SubDevice *> &island : peer_islands) {
+      SubDevice *owner_sub = find_suitable_mem_device(existing_key, island);
+      mem.device = owner_sub->device.get();
+      mem.device_pointer = (existing_key) ? owner_sub->ptr_map[existing_key] : 0;
+      mem.device_size = existing_size;
+
+      if (!owner_sub->device->is_shared(
+              mem.shared_pointer, mem.device_pointer, owner_sub->device.get()))
+      {
+        owner_sub->device->mem_move_to_host(mem);
+        owner_sub->ptr_map[key] = mem.device_pointer;
+
+        /* Need to create texture objects and update pointer in kernel globals on all devices */
+        for (SubDevice *island_sub : island) {
+          if (island_sub != owner_sub) {
+            island_sub->device->mem_move_to_host(mem);
+          }
+        }
+      }
+    }
+
+    mem.device = this;
+    mem.device_pointer = key;
+    stats.mem_alloc(mem.device_size - existing_size);
+  }
+
+  bool is_shared(const void *shared_pointer, const device_ptr key, Device *sub_device) override
+  {
+    if (key == 0) {
+      return false;
+    }
+
+    for (const SubDevice &sub : devices) {
+      if (sub.device.get() == sub_device) {
+        return sub_device->is_shared(shared_pointer, sub.ptr_map.at(key), sub_device);
+      }
+    }
+
+    assert(!"is_shared failed to find matching device");
+    return false;
+  }
+
+  void mem_copy_from(
+      device_memory &mem, const size_t y, size_t w, const size_t h, size_t elem) override
   {
     device_ptr key = mem.device_pointer;
-    size_t i = 0, sub_h = h / devices.size();
+    const size_t sub_h = h / devices.size();
+    size_t i = 0;
 
-    foreach (SubDevice &sub, devices) {
+    for (SubDevice &sub : devices) {
       size_t sy = y + i * sub_h;
       size_t sh = (i == (size_t)devices.size() - 1) ? h - sub_h * i : sub_h;
 
       SubDevice *owner_sub = find_matching_mem_device(key, sub);
-      mem.device = owner_sub->device;
+      mem.device = owner_sub->device.get();
       mem.device_pointer = owner_sub->ptr_map[key];
 
       owner_sub->device->mem_copy_from(mem, sy, w, sh, elem);
@@ -393,9 +496,9 @@ class MultiDevice : public Device {
     device_ptr key = (existing_key) ? existing_key : unique_key++;
     size_t existing_size = mem.device_size;
 
-    foreach (const vector<SubDevice *> &island, peer_islands) {
+    for (const vector<SubDevice *> &island : peer_islands) {
       SubDevice *owner_sub = find_suitable_mem_device(existing_key, island);
-      mem.device = owner_sub->device;
+      mem.device = owner_sub->device.get();
       mem.device_pointer = (existing_key) ? owner_sub->ptr_map[existing_key] : 0;
       mem.device_size = existing_size;
 
@@ -414,9 +517,9 @@ class MultiDevice : public Device {
     size_t existing_size = mem.device_size;
 
     /* Free memory that was allocated for all devices (see above) on each device */
-    foreach (const vector<SubDevice *> &island, peer_islands) {
+    for (const vector<SubDevice *> &island : peer_islands) {
       SubDevice *owner_sub = find_matching_mem_device(key, *island.front());
-      mem.device = owner_sub->device;
+      mem.device = owner_sub->device.get();
       mem.device_pointer = owner_sub->ptr_map[key];
       mem.device_size = existing_size;
 
@@ -425,7 +528,7 @@ class MultiDevice : public Device {
 
       if (mem.type == MEM_TEXTURE) {
         /* Free texture objects on all devices */
-        foreach (SubDevice *island_sub, island) {
+        for (SubDevice *island_sub : island) {
           if (island_sub != owner_sub) {
             island_sub->device->mem_free(mem);
           }
@@ -439,18 +542,19 @@ class MultiDevice : public Device {
     stats.mem_free(existing_size);
   }
 
-  void const_copy_to(const char *name, void *host, size_t size) override
+  void const_copy_to(const char *name, void *host, const size_t size) override
   {
-    foreach (SubDevice &sub, devices)
+    for (SubDevice &sub : devices) {
       sub.device->const_copy_to(name, host, size);
+    }
   }
 
   int device_number(Device *sub_device) override
   {
     int i = 0;
 
-    foreach (SubDevice &sub, devices) {
-      if (sub.device == sub_device) {
+    for (SubDevice &sub : devices) {
+      if (sub.device.get() == sub_device) {
         return i;
       }
       i++;
@@ -459,17 +563,20 @@ class MultiDevice : public Device {
     return -1;
   }
 
-  virtual void foreach_device(const function<void(Device *)> &callback) override
+  void foreach_device(const std::function<void(Device *)> &callback) override
   {
-    foreach (SubDevice &sub, devices) {
+    for (SubDevice &sub : devices) {
       sub.device->foreach_device(callback);
     }
   }
 };
 
-Device *device_multi_create(const DeviceInfo &info, Stats &stats, Profiler &profiler)
+unique_ptr<Device> device_multi_create(const DeviceInfo &info,
+                                       Stats &stats,
+                                       Profiler &profiler,
+                                       bool headless)
 {
-  return new MultiDevice(info, stats, profiler);
+  return make_unique<MultiDevice>(info, stats, profiler, headless);
 }
 
 CCL_NAMESPACE_END
