@@ -10,8 +10,8 @@
  * More work is needed for logging drawing to work properly.
  *
  * - Use GPU_matrix API.
- * - Replace old OpenGL calls to glColor, etc with 'imm' API.
- * - Investigate BLF font flushing (UI_widgetbase_draw_cache_flush) which is currently disabled.
+ * - Replace old OpenGL calls to `glColor`, etc with `imm` API.
+ * - Investigate BLF font flushing (`UI_widgetbase_draw_cache_flush`) which is currently disabled.
  */
 
 #ifdef _MSC_VER
@@ -30,7 +30,7 @@
 
 #include "GHOST_C-api.h"
 
-#include "BLF_api.h"
+#include "BLF_api.hh"
 
 #include "Basic.h"
 #include "EventToBuf.h"
@@ -40,9 +40,9 @@
 #include "WindowData.h"
 
 /* GPU API. */
-#include "GPU_context.h"
-#include "GPU_immediate.h"
-#include "GPU_init_exit.h"
+#include "GPU_context.hh"
+#include "GPU_immediate.hh"
+#include "GPU_init_exit.hh"
 
 extern int datatoc_bfont_ttf_size;
 extern char const datatoc_bfont_ttf[];
@@ -874,6 +874,7 @@ MultiTestApp *multitestapp_new(void)
   if (!app->sys) {
     fatal("Unable to create ghost system");
   }
+  GPU_backend_ghost_system_set(app->sys);
 
   if (!GHOST_AddEventConsumer(app->sys, consumer)) {
     fatal("Unable to add multitest event consumer ");
