@@ -24,6 +24,7 @@
 #include "DNA_texture_types.h"
 
 #include "BLI_ghash.h"
+#include "BLI_listbase.h"
 #include "BLI_math_base_safe.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_rotation.h"
@@ -56,8 +57,7 @@
 
 EffectorWeights *BKE_effector_add_weights(Collection *collection)
 {
-  EffectorWeights *weights = static_cast<EffectorWeights *>(
-      MEM_callocN(sizeof(EffectorWeights), "EffectorWeights"));
+  EffectorWeights *weights = MEM_callocN<EffectorWeights>("EffectorWeights");
   for (int i = 0; i < NUM_PFIELD_TYPES; i++) {
     weights->weight[i] = 1.0f;
   }
@@ -72,7 +72,7 @@ PartDeflect *BKE_partdeflect_new(int type)
 {
   PartDeflect *pd;
 
-  pd = static_cast<PartDeflect *>(MEM_callocN(sizeof(PartDeflect), "PartDeflect"));
+  pd = MEM_callocN<PartDeflect>("PartDeflect");
 
   pd->forcefield = type;
   pd->pdef_sbdamp = 0.1f;
@@ -166,8 +166,7 @@ static void add_effector_relation(ListBase *relations,
                                   ParticleSystem *psys,
                                   PartDeflect *pd)
 {
-  EffectorRelation *relation = static_cast<EffectorRelation *>(
-      MEM_callocN(sizeof(EffectorRelation), "EffectorRelation"));
+  EffectorRelation *relation = MEM_callocN<EffectorRelation>("EffectorRelation");
   relation->ob = ob;
   relation->psys = psys;
   relation->pd = pd;
@@ -183,11 +182,10 @@ static void add_effector_evaluation(ListBase **effectors,
                                     PartDeflect *pd)
 {
   if (*effectors == nullptr) {
-    *effectors = static_cast<ListBase *>(MEM_callocN(sizeof(ListBase), "effector effectors"));
+    *effectors = MEM_callocN<ListBase>("effector effectors");
   }
 
-  EffectorCache *eff = static_cast<EffectorCache *>(
-      MEM_callocN(sizeof(EffectorCache), "EffectorCache"));
+  EffectorCache *eff = MEM_callocN<EffectorCache>("EffectorCache");
   eff->depsgraph = depsgraph;
   eff->scene = scene;
   eff->ob = ob;
@@ -208,8 +206,7 @@ ListBase *BKE_effector_relations_create(Depsgraph *depsgraph,
   const bool for_render = (DEG_get_mode(depsgraph) == DAG_EVAL_RENDER);
   const int base_flag = (for_render) ? BASE_ENABLED_RENDER : BASE_ENABLED_VIEWPORT;
 
-  ListBase *relations = static_cast<ListBase *>(
-      MEM_callocN(sizeof(ListBase), "effector relations"));
+  ListBase *relations = MEM_callocN<ListBase>("effector relations");
 
   for (; base; base = base->next) {
     if (!(base->flag & base_flag)) {
@@ -322,7 +319,7 @@ ListBase *BKE_effectors_create(Depsgraph *depsgraph,
 
   LISTBASE_FOREACH (EffectorRelation *, relation, relations) {
     /* Get evaluated object. */
-    Object *ob = (Object *)DEG_get_evaluated_id(depsgraph, &relation->ob->id);
+    Object *ob = DEG_get_evaluated(depsgraph, relation->ob);
 
     if (relation->psys) {
       /* Get evaluated particle system. */
@@ -1262,8 +1259,7 @@ void BKE_sim_debug_data_set_enabled(bool enable)
 {
   if (enable) {
     if (!_sim_debug_data) {
-      _sim_debug_data = static_cast<SimDebugData *>(
-          MEM_callocN(sizeof(SimDebugData), "sim debug data"));
+      _sim_debug_data = MEM_callocN<SimDebugData>("sim debug data");
       _sim_debug_data->gh = BLI_ghash_new(
           debug_element_hash, debug_element_compare, "sim debug element hash");
     }
@@ -1323,8 +1319,7 @@ void BKE_sim_debug_data_add_element(int type,
     }
   }
 
-  elem = static_cast<SimDebugElement *>(
-      MEM_callocN(sizeof(SimDebugElement), "sim debug data element"));
+  elem = MEM_callocN<SimDebugElement>("sim debug data element");
   elem->type = type;
   elem->category_hash = category_hash;
   elem->hash = hash;
