@@ -239,15 +239,15 @@ void face_angles_calc(const Span<float3> vert_positions,
 
 bool BKE_mesh_center_median(const Mesh *mesh, float r_cent[3])
 {
-  const Span<float3> positions = mesh->vert_positions();
-  zero_v3(r_cent);
-  for (const int i : positions.index_range()) {
-    add_v3_v3(r_cent, positions[i]);
-  }
+  float3 center = blender::array_utils::compute_sum<float3>(mesh->vert_positions());
+
   /* otherwise we get NAN for 0 verts */
   if (mesh->verts_num) {
-    mul_v3_fl(r_cent, 1.0f / float(mesh->verts_num));
+    mul_v3_fl(center, 1.0 / float(mesh->verts_num));
   }
+
+  copy_v3_v3(r_cent, center);
+
   return (mesh->verts_num != 0);
 }
 
@@ -723,7 +723,7 @@ void BKE_mesh_calc_relative_deform(const int *face_offsets,
 {
   const blender::OffsetIndices<int> faces({face_offsets, faces_num + 1});
 
-  int *vert_accum = (int *)MEM_calloc_arrayN(size_t(totvert), sizeof(*vert_accum), __func__);
+  int *vert_accum = MEM_calloc_arrayN<int>(totvert, __func__);
 
   memset(vert_cos_new, '\0', sizeof(*vert_cos_new) * size_t(totvert));
 
