@@ -24,7 +24,7 @@
 #include "COM_ocio_color_space_conversion_shader.hh"
 #include "COM_result.hh"
 
-#if defined(WITH_OCIO)
+#if defined(WITH_OPENCOLORIO)
 #  include <OpenColorIO/OpenColorIO.h>
 #endif
 
@@ -55,7 +55,7 @@ bool operator==(const OCIOColorSpaceConversionShaderKey &a,
  * GPU Shader Creator.
  */
 
-#if defined(WITH_OCIO)
+#if defined(WITH_OPENCOLORIO)
 
 namespace OCIO = OCIO_NAMESPACE;
 using namespace blender::gpu::shader;
@@ -121,7 +121,7 @@ class GPUShaderCreator : public OCIO::GpuShaderCreator {
     /* Don't use the name argument directly since ShaderCreateInfo only stores references to
      * resource names, instead, use the name that is stored in resource_names_. */
     std::string &resource_name = *resource_names_[resource_names_.size() - 1];
-    shader_create_info_.push_constant(Type::FLOAT, resource_name);
+    shader_create_info_.push_constant(Type::float_t, resource_name);
 
     float_uniforms_.add(resource_name, get_double);
 
@@ -140,7 +140,7 @@ class GPUShaderCreator : public OCIO::GpuShaderCreator {
     /* Don't use the name argument directly since ShaderCreateInfo only stores references to
      * resource names, instead, use the name that is stored in resource_names_. */
     const std::string &resource_name = *resource_names_[resource_names_.size() - 1];
-    shader_create_info_.push_constant(Type::BOOL, resource_name);
+    shader_create_info_.push_constant(Type::bool_t, resource_name);
 
     boolean_uniforms_.add(name, get_bool);
 
@@ -159,7 +159,7 @@ class GPUShaderCreator : public OCIO::GpuShaderCreator {
     /* Don't use the name argument directly since ShaderCreateInfo only stores references to
      * resource names, instead, use the name that is stored in resource_names_. */
     std::string &resource_name = *resource_names_[resource_names_.size() - 1];
-    shader_create_info_.push_constant(Type::VEC3, resource_name);
+    shader_create_info_.push_constant(Type::float3_t, resource_name);
 
     vector_uniforms_.add(resource_name, get_float3);
 
@@ -231,9 +231,8 @@ class GPUShaderCreator : public OCIO::GpuShaderCreator {
     const std::string &resource_name = *resource_names_[resource_names_.size() - 1];
 
     GPUTexture *texture;
-    const ResultType result_type = (channel == TEXTURE_RGB_CHANNEL) ? ResultType::Float3 :
-                                                                      ResultType::Float;
-    const eGPUTextureFormat texture_format = Result::gpu_texture_format(result_type, precision_);
+    const eGPUTextureFormat base_format = (channel == TEXTURE_RGB_CHANNEL) ? GPU_RGB32F : GPU_R32F;
+    const eGPUTextureFormat texture_format = Result::gpu_texture_format(base_format, precision_);
     /* A height of 1 indicates a 1D texture according to the OCIO API. */
 #  if OCIO_VERSION_HEX >= 0x02030000
     if (dimensions == OCIO::GpuShaderDesc::TEXTURE_1D)
@@ -243,12 +242,12 @@ class GPUShaderCreator : public OCIO::GpuShaderCreator {
     {
       texture = GPU_texture_create_1d(
           texture_name, width, 1, texture_format, GPU_TEXTURE_USAGE_SHADER_READ, values);
-      shader_create_info_.sampler(textures_.size() + 1, ImageType::FLOAT_1D, resource_name);
+      shader_create_info_.sampler(textures_.size() + 1, ImageType::Float1D, resource_name);
     }
     else {
       texture = GPU_texture_create_2d(
           texture_name, width, height, 1, texture_format, GPU_TEXTURE_USAGE_SHADER_READ, values);
-      shader_create_info_.sampler(textures_.size() + 1, ImageType::FLOAT_2D, resource_name);
+      shader_create_info_.sampler(textures_.size() + 1, ImageType::Float2D, resource_name);
     }
     GPU_texture_filter_mode(texture, interpolation != OCIO::INTERP_NEAREST);
 
@@ -269,17 +268,16 @@ class GPUShaderCreator : public OCIO::GpuShaderCreator {
     /* Don't use the name argument directly since ShaderCreateInfo only stores references to
      * resource names, instead, use the name that is stored in resource_names_. */
     const std::string &resource_name = *resource_names_[resource_names_.size() - 1];
-    shader_create_info_.sampler(textures_.size() + 1, ImageType::FLOAT_3D, resource_name);
+    shader_create_info_.sampler(textures_.size() + 1, ImageType::Float3D, resource_name);
 
-    GPUTexture *texture = GPU_texture_create_3d(
-        texture_name,
-        size,
-        size,
-        size,
-        1,
-        Result::gpu_texture_format(ResultType::Float3, precision_),
-        GPU_TEXTURE_USAGE_SHADER_READ,
-        values);
+    GPUTexture *texture = GPU_texture_create_3d(texture_name,
+                                                size,
+                                                size,
+                                                size,
+                                                1,
+                                                Result::gpu_texture_format(GPU_RGB32F, precision_),
+                                                GPU_TEXTURE_USAGE_SHADER_READ,
+                                                values);
     GPU_texture_filter_mode(texture, interpolation != OCIO::INTERP_NEAREST);
 
     textures_.add(sampler_name, texture);
@@ -308,11 +306,11 @@ class GPUShaderCreator : public OCIO::GpuShaderCreator {
     GpuShaderCreator::finalize();
 
     shader_create_info_.local_group_size(16, 16);
-    shader_create_info_.sampler(0, ImageType::FLOAT_2D, input_sampler_name());
+    shader_create_info_.sampler(0, ImageType::Float2D, input_sampler_name());
     shader_create_info_.image(0,
                               Result::gpu_texture_format(ResultType::Color, precision_),
-                              Qualifier::WRITE,
-                              ImageType::FLOAT_2D,
+                              Qualifier::write,
+                              ImageReadWriteType::Float2D,
                               output_image_name());
     shader_create_info_.compute_source("gpu_shader_compositor_ocio_processor.glsl");
     shader_create_info_.compute_source_generated += shader_code_;
@@ -488,7 +486,7 @@ OCIOColorSpaceConversionShader::OCIOColorSpaceConversionShader(Context &context,
    * processor. */
   shader_creator_ = GPUShaderCreator::Create(context.get_precision());
 
-#if defined(WITH_OCIO)
+#if defined(WITH_OPENCOLORIO)
   /* Get a GPU processor that transforms the source color space to the target color space. */
   try {
     OCIO::ConstConfigRcPtr config = OCIO::GetCurrentConfig();
@@ -545,7 +543,7 @@ OCIOColorSpaceConversionShader &OCIOColorSpaceConversionShaderContainer::get(Con
                                                                              std::string source,
                                                                              std::string target)
 {
-#if defined(WITH_OCIO)
+#if defined(WITH_OPENCOLORIO)
   /* Use the config cache ID in the cache key in case the configuration changed at runtime. */
   std::string config_cache_id = OCIO::GetCurrentConfig()->getCacheID();
 #else
