@@ -35,14 +35,14 @@ void VolumePass::object_sync_volume(Manager &manager,
 {
   Object *ob = ob_ref.object;
   /* Create 3D textures. */
-  Volume *volume = static_cast<Volume *>(ob->data);
-  BKE_volume_load(volume, G.main);
-  const bke::VolumeGridData *volume_grid = BKE_volume_grid_active_get_for_read(volume);
+  Volume &volume = DRW_object_get_data_for_drawing<Volume>(*ob);
+  BKE_volume_load(&volume, G.main);
+  const bke::VolumeGridData *volume_grid = BKE_volume_grid_active_get_for_read(&volume);
   if (volume_grid == nullptr) {
     return;
   }
 
-  DRWVolumeGrid *grid = DRW_volume_batch_cache_get_grid(volume, volume_grid);
+  DRWVolumeGrid *grid = DRW_volume_batch_cache_get_grid(&volume, volume_grid);
   if (grid == nullptr) {
     return;
   }
@@ -51,28 +51,28 @@ void VolumePass::object_sync_volume(Manager &manager,
 
   PassMain::Sub &sub_ps = ps_.sub("Volume Object SubPass");
 
-  const bool use_slice = (volume->display.axis_slice_method == AXIS_SLICE_SINGLE);
+  const bool use_slice = (volume.display.axis_slice_method == AXIS_SLICE_SINGLE);
 
-  sub_ps.shader_set(ShaderCache::get().volume_get(
-      false, volume->display.interpolation_method, false, use_slice));
+  sub_ps.shader_set(
+      ShaderCache::get().volume_get(false, volume.display.interpolation_method, false, use_slice));
   sub_ps.push_constant("do_depth_test", scene_state.shading.type >= OB_SOLID);
 
-  const float density_scale = volume->display.density *
-                              BKE_volume_density_scale(volume, ob->object_to_world().ptr());
+  const float density_fac = volume.display.density *
+                            BKE_volume_density_scale(&volume, ob->object_to_world().ptr());
 
-  sub_ps.bind_texture("depthBuffer", &resources.depth_tx);
+  sub_ps.bind_texture("depth_buffer", &resources.depth_tx);
   sub_ps.bind_texture("stencil_tx", &stencil_tx_);
-  sub_ps.bind_texture("densityTexture", grid->texture);
+  sub_ps.bind_texture("density_tx", grid->texture);
   /* TODO: implement shadow texture, see manta_smoke_calc_transparency. */
-  sub_ps.bind_texture("shadowTexture", dummy_shadow_tx_);
-  sub_ps.push_constant("activeColor", color);
-  sub_ps.push_constant("densityScale", density_scale);
-  sub_ps.push_constant("volumeObjectToTexture", float4x4(grid->object_to_texture));
-  sub_ps.push_constant("volumeTextureToObject", float4x4(grid->texture_to_object));
+  sub_ps.bind_texture("shadow_tx", dummy_shadow_tx_);
+  sub_ps.push_constant("active_color", color);
+  sub_ps.push_constant("density_fac", density_fac);
+  sub_ps.push_constant("volume_object_to_texture", float4x4(grid->object_to_texture));
+  sub_ps.push_constant("volume_texture_to_object", float4x4(grid->texture_to_object));
 
   if (use_slice) {
     draw_slice_ps(
-        manager, sub_ps, ob_ref, volume->display.slice_axis, volume->display.slice_depth);
+        manager, resources, sub_ps, ob_ref, volume.display.slice_axis, volume.display.slice_depth);
   }
   else {
     float4x4 texture_to_world = ob->object_to_world() * float4x4(grid->texture_to_object);
@@ -82,7 +82,8 @@ void VolumePass::object_sync_volume(Manager &manager,
     GPU_texture_get_mipmap_size(grid->texture, 0, resolution);
     float3 slice_count = float3(resolution) * 5.0f;
 
-    draw_volume_ps(manager, sub_ps, ob_ref, scene_state.sample, slice_count, world_size);
+    draw_volume_ps(
+        manager, resources, sub_ps, ob_ref, scene_state.sample, slice_count, world_size);
   }
 }
 
@@ -134,44 +135,43 @@ void VolumePass::object_sync_modifier(Manager &manager,
                                FLUID_DOMAIN_FIELD_PHI_OUT,
                                FLUID_DOMAIN_FIELD_PHI_OBSTACLE);
 
-    sub_ps.push_constant("showFlags", show_flags);
-    sub_ps.push_constant("showPressure", show_pressure);
-    sub_ps.push_constant("showPhi", show_phi);
-    sub_ps.push_constant("gridScale", settings.grid_scale);
+    sub_ps.push_constant("show_flags", show_flags);
+    sub_ps.push_constant("show_pressure", show_pressure);
+    sub_ps.push_constant("show_phi", show_phi);
+    sub_ps.push_constant("grid_scale", settings.grid_scale);
 
     if (show_flags) {
-      sub_ps.bind_texture("flagTexture", settings.tex_field);
+      sub_ps.bind_texture("flag_tx", settings.tex_field);
     }
     else {
-      sub_ps.bind_texture("densityTexture", settings.tex_field);
+      sub_ps.bind_texture("density_tx", settings.tex_field);
     }
 
     if (!show_flags && !show_pressure && !show_phi) {
-      sub_ps.bind_texture("transferTexture", settings.tex_coba);
+      sub_ps.bind_texture("transfer_tx", settings.tex_coba);
     }
   }
   else {
     bool use_constant_color = ((settings.active_fields & FLUID_DOMAIN_ACTIVE_COLORS) == 0 &&
                                (settings.active_fields & FLUID_DOMAIN_ACTIVE_COLOR_SET) != 0);
 
-    sub_ps.push_constant("activeColor",
+    sub_ps.push_constant("active_color",
                          use_constant_color ? float3(settings.active_color) : float3(1));
 
-    sub_ps.bind_texture("densityTexture",
+    sub_ps.bind_texture("density_tx",
                         settings.tex_color ? settings.tex_color : settings.tex_density);
-    sub_ps.bind_texture("flameTexture",
-                        settings.tex_flame ? settings.tex_flame : dummy_volume_tx_);
-    sub_ps.bind_texture("flameColorTexture",
+    sub_ps.bind_texture("flame_tx", settings.tex_flame ? settings.tex_flame : dummy_volume_tx_);
+    sub_ps.bind_texture("flame_color_tx",
                         settings.tex_flame ? settings.tex_flame_coba : dummy_coba_tx_);
-    sub_ps.bind_texture("shadowTexture", settings.tex_shadow);
+    sub_ps.bind_texture("shadow_tx", settings.tex_shadow);
   }
 
-  sub_ps.push_constant("densityScale", 10.0f * settings.display_thickness);
-  sub_ps.bind_texture("depthBuffer", &resources.depth_tx);
+  sub_ps.push_constant("density_fac", 10.0f * settings.display_thickness);
+  sub_ps.bind_texture("depth_buffer", &resources.depth_tx);
   sub_ps.bind_texture("stencil_tx", &stencil_tx_);
 
   if (use_slice) {
-    draw_slice_ps(manager, sub_ps, ob_ref, settings.slice_axis, settings.slice_depth);
+    draw_slice_ps(manager, resources, sub_ps, ob_ref, settings.slice_axis, settings.slice_depth);
   }
   else {
     float3 world_size;
@@ -179,7 +179,8 @@ void VolumePass::object_sync_modifier(Manager &manager,
 
     float3 slice_count = float3(settings.res) * std::max(0.001f, settings.slice_per_voxel);
 
-    draw_volume_ps(manager, sub_ps, ob_ref, scene_state.sample, slice_count, world_size);
+    draw_volume_ps(
+        manager, resources, sub_ps, ob_ref, scene_state.sample, slice_count, world_size);
   }
 }
 
@@ -189,15 +190,19 @@ void VolumePass::draw(Manager &manager, View &view, SceneResources &resources)
     return;
   }
 
-  stencil_tx_ = resources.stencil_view.extract(manager, resources.depth_tx);
+  stencil_tx_ = resources.depth_tx.ptr()->stencil_view();
 
   fb_.ensure(GPU_ATTACHMENT_NONE, GPU_ATTACHMENT_TEXTURE(resources.color_tx));
   fb_.bind();
   manager.submit(ps_, view);
 }
 
-void VolumePass::draw_slice_ps(
-    Manager &manager, PassMain::Sub &ps, ObjectRef &ob_ref, int slice_axis_enum, float slice_depth)
+void VolumePass::draw_slice_ps(Manager &manager,
+                               SceneResources &resources,
+                               PassMain::Sub &ps,
+                               ObjectRef &ob_ref,
+                               int slice_axis_enum,
+                               float slice_depth)
 {
   float4x4 view_mat_inv = blender::draw::View::default_get().viewinv();
 
@@ -210,15 +215,16 @@ void VolumePass::draw_slice_ps(
   /* 0.05f to achieve somewhat the same opacity as the full view. */
   float step_length = std::max(1e-16f, dimensions[axis] * 0.05f);
 
-  ps.state_set(DRW_STATE_WRITE_COLOR | DRW_STATE_BLEND_ALPHA_PREMUL);
-  ps.push_constant("slicePosition", slice_depth);
-  ps.push_constant("sliceAxis", axis);
-  ps.push_constant("stepLength", step_length);
+  ps.state_set(DRW_STATE_WRITE_COLOR | DRW_STATE_BLEND_ALPHA_PREMUL | DRW_STATE_CULL_FRONT);
+  ps.push_constant("slice_position", slice_depth);
+  ps.push_constant("slice_axis", axis);
+  ps.push_constant("step_length", step_length);
 
-  ps.draw(DRW_cache_quad_get(), manager.resource_handle(ob_ref));
+  ps.draw(resources.volume_cube_batch, manager.resource_handle(ob_ref));
 }
 
 void VolumePass::draw_volume_ps(Manager &manager,
+                                SceneResources &resources,
                                 PassMain::Sub &ps,
                                 ObjectRef &ob_ref,
                                 int taa_sample,
@@ -232,11 +238,11 @@ void VolumePass::draw_volume_ps(Manager &manager,
   float step_length = math::length((1.0f / slice_count) * world_size);
 
   ps.state_set(DRW_STATE_WRITE_COLOR | DRW_STATE_BLEND_ALPHA_PREMUL | DRW_STATE_CULL_FRONT);
-  ps.push_constant("samplesLen", max_slice);
-  ps.push_constant("stepLength", step_length);
-  ps.push_constant("noiseOfs", float(noise_offset));
+  ps.push_constant("samples_len", max_slice);
+  ps.push_constant("step_length", step_length);
+  ps.push_constant("noise_ofs", float(noise_offset));
 
-  ps.draw(DRW_cache_cube_get(), manager.resource_handle(ob_ref));
+  ps.draw(resources.volume_cube_batch, manager.resource_handle(ob_ref));
 }
 
 }  // namespace blender::workbench
