@@ -601,8 +601,7 @@ static void insert_bezt_to_nurb(Nurb *nu, const CutData *data, Curve *cu)
 {
   EditNurb *editnurb = cu->editnurb;
 
-  BezTriple *new_bezt_array = (BezTriple *)MEM_mallocN((nu->pntsu + 1) * sizeof(BezTriple),
-                                                       __func__);
+  BezTriple *new_bezt_array = MEM_malloc_arrayN<BezTriple>((nu->pntsu + 1), __func__);
   const int index = data->bezt_index + 1;
   /* Copy all control points before the cut to the new memory. */
   ED_curve_beztcpy(editnurb, new_bezt_array, nu->bezt, index);
@@ -657,7 +656,7 @@ static void insert_bp_to_nurb(Nurb *nu, const CutData *data, Curve *cu)
 {
   EditNurb *editnurb = cu->editnurb;
 
-  BPoint *new_bp_array = (BPoint *)MEM_mallocN((nu->pntsu + 1) * sizeof(BPoint), __func__);
+  BPoint *new_bp_array = MEM_malloc_arrayN<BPoint>((nu->pntsu + 1), __func__);
   const int index = data->bp_index + 1;
   /* Copy all control points before the cut to the new memory. */
   ED_curve_bpcpy(editnurb, new_bp_array, nu->bp, index);
@@ -756,7 +755,7 @@ static void update_cut_data_for_nurb(
 
   if (nu->type == CU_BEZIER) {
     for (int i = 0; i < end; i++) {
-      float *points = static_cast<float *>(MEM_mallocN(sizeof(float[3]) * (resolu + 1), __func__));
+      float *points = MEM_malloc_arrayN<float>(3 * (resolu + 1), __func__);
 
       const BezTriple *bezt1 = nu->bezt + i;
       const BezTriple *bezt2 = nu->bezt + (i + 1) % nu->pntsu;
@@ -945,8 +944,7 @@ static void extrude_vertices_from_selected_endpoints(EditNurb *editnurb,
       const bool last_sel = BEZT_ISSEL_ANY(last_bezt) && nu1->pntsu > 1;
       if (first_sel) {
         if (last_sel) {
-          BezTriple *new_bezt = (BezTriple *)MEM_mallocN((nu1->pntsu + 2) * sizeof(BezTriple),
-                                                         __func__);
+          BezTriple *new_bezt = MEM_malloc_arrayN<BezTriple>((nu1->pntsu + 2), __func__);
           ED_curve_beztcpy(editnurb, new_bezt, nu1->bezt, 1);
           ED_curve_beztcpy(editnurb, new_bezt + nu1->pntsu + 1, last_bezt, 1);
           BEZT_DESEL_ALL(nu1->bezt);
@@ -967,8 +965,7 @@ static void extrude_vertices_from_selected_endpoints(EditNurb *editnurb,
           BEZT_SEL_IDX(new_bezt + (nu1->pntsu - 1), 2);
         }
         else {
-          BezTriple *new_bezt = (BezTriple *)MEM_mallocN((nu1->pntsu + 1) * sizeof(BezTriple),
-                                                         __func__);
+          BezTriple *new_bezt = MEM_malloc_arrayN<BezTriple>((nu1->pntsu + 1), __func__);
           ED_curve_beztcpy(editnurb, new_bezt, nu1->bezt, 1);
           BEZT_DESEL_ALL(nu1->bezt);
           ED_curve_beztcpy(editnurb, new_bezt + 1, nu1->bezt, nu1->pntsu);
@@ -985,8 +982,7 @@ static void extrude_vertices_from_selected_endpoints(EditNurb *editnurb,
         cu->actvert = 0;
       }
       else if (last_sel) {
-        BezTriple *new_bezt = (BezTriple *)MEM_mallocN((nu1->pntsu + 1) * sizeof(BezTriple),
-                                                       __func__);
+        BezTriple *new_bezt = MEM_malloc_arrayN<BezTriple>((nu1->pntsu + 1), __func__);
         ED_curve_beztcpy(editnurb, new_bezt + nu1->pntsu, last_bezt, 1);
         BEZT_DESEL_ALL(last_bezt);
         ED_curve_beztcpy(editnurb, new_bezt, nu1->bezt, nu1->pntsu);
@@ -1008,7 +1004,7 @@ static void extrude_vertices_from_selected_endpoints(EditNurb *editnurb,
       const bool last_sel = last_bp->f1 & SELECT && nu1->pntsu > 1;
       if (first_sel) {
         if (last_sel) {
-          BPoint *new_bp = (BPoint *)MEM_mallocN((nu1->pntsu + 2) * sizeof(BPoint), __func__);
+          BPoint *new_bp = MEM_malloc_arrayN<BPoint>((nu1->pntsu + 2), __func__);
           ED_curve_bpcpy(editnurb, new_bp, nu1->bp, 1);
           ED_curve_bpcpy(editnurb, new_bp + nu1->pntsu + 1, last_bp, 1);
           nu1->bp->f1 &= ~SELECT;
@@ -1021,7 +1017,7 @@ static void extrude_vertices_from_selected_endpoints(EditNurb *editnurb,
           nu1->pntsu += 2;
         }
         else {
-          BPoint *new_bp = (BPoint *)MEM_mallocN((nu1->pntsu + 1) * sizeof(BPoint), __func__);
+          BPoint *new_bp = MEM_malloc_arrayN<BPoint>((nu1->pntsu + 1), __func__);
           ED_curve_bpcpy(editnurb, new_bp, nu1->bp, 1);
           nu1->bp->f1 &= ~SELECT;
           ED_curve_bpcpy(editnurb, new_bp + 1, nu1->bp, nu1->pntsu);
@@ -1035,7 +1031,7 @@ static void extrude_vertices_from_selected_endpoints(EditNurb *editnurb,
         cu->actvert = 0;
       }
       else if (last_sel) {
-        BPoint *new_bp = (BPoint *)MEM_mallocN((nu1->pntsu + 1) * sizeof(BPoint), __func__);
+        BPoint *new_bp = MEM_malloc_arrayN<BPoint>((nu1->pntsu + 1), __func__);
         ED_curve_bpcpy(editnurb, new_bp, nu1->bp, nu1->pntsu);
         ED_curve_bpcpy(editnurb, new_bp + nu1->pntsu, last_bp, 1);
         last_bp->f1 &= ~SELECT;
@@ -1161,8 +1157,7 @@ static bool is_spline_nearby(ViewContext *vc,
     if (cd.nurb && (cd.nurb->type == CU_BEZIER) && RNA_boolean_get(op->ptr, "move_segment")) {
       MoveSegmentData *seg_data;
       CurvePenData *cpd = (CurvePenData *)(op->customdata);
-      cpd->msd = seg_data = static_cast<MoveSegmentData *>(
-          MEM_callocN(sizeof(MoveSegmentData), __func__));
+      cpd->msd = seg_data = MEM_callocN<MoveSegmentData>(__func__);
       seg_data->bezt_index = cd.bezt_index;
       seg_data->nu = cd.nurb;
       seg_data->t = cd.parameter;
@@ -1553,7 +1548,7 @@ wmKeyMap *curve_pen_modal_keymap(wmKeyConfig *keyconf)
   return keymap;
 }
 
-static int curve_pen_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus curve_pen_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   Object *obedit = CTX_data_edit_object(C);
@@ -1567,11 +1562,12 @@ static int curve_pen_modal(bContext *C, wmOperator *op, const wmEvent *event)
   BPoint *bp = nullptr;
   Nurb *nu = nullptr;
 
-  SelectPick_Params params{};
-  params.sel_op = SEL_OP_SET;
-  params.deselect_all = false;
+  const SelectPick_Params params = {
+      /*sel_op*/ SEL_OP_SET,
+      /*deselect_all*/ false,
+  };
 
-  int ret = OPERATOR_RUNNING_MODAL;
+  wmOperatorStatus ret = OPERATOR_RUNNING_MODAL;
 
   /* Distance threshold for mouse clicks to affect the spline or its points */
   const float mval_fl[2] = {float(event->mval[0]), float(event->mval[1])};
@@ -1590,8 +1586,7 @@ static int curve_pen_modal(bContext *C, wmOperator *op, const wmEvent *event)
 
   CurvePenData *cpd;
   if (op->customdata == nullptr) {
-    op->customdata = cpd = static_cast<CurvePenData *>(
-        MEM_callocN(sizeof(CurvePenData), __func__));
+    op->customdata = cpd = MEM_callocN<CurvePenData>(__func__);
   }
   else {
     cpd = (CurvePenData *)(op->customdata);
@@ -1663,7 +1658,7 @@ static int curve_pen_modal(bContext *C, wmOperator *op, const wmEvent *event)
   else if (ELEM(event->type, LEFTMOUSE)) {
     if (ELEM(event->val, KM_RELEASE, KM_DBL_CLICK)) {
       if (delete_point && !cpd->new_point && !cpd->dragging) {
-        if (ED_curve_editnurb_select_pick(C, event->mval, threshold_dist_px, &params)) {
+        if (ED_curve_editnurb_select_pick(C, event->mval, threshold_dist_px, params)) {
           cpd->changed = delete_point_under_mouse(&vc, event);
         }
       }
@@ -1724,7 +1719,7 @@ static int curve_pen_modal(bContext *C, wmOperator *op, const wmEvent *event)
           }
         }
         else if (select_point) {
-          ED_curve_editnurb_select_pick(C, event->mval, threshold_dist_px, &params);
+          ED_curve_editnurb_select_pick(C, event->mval, threshold_dist_px, params);
         }
       }
 
@@ -1743,7 +1738,7 @@ static int curve_pen_modal(bContext *C, wmOperator *op, const wmEvent *event)
   return ret;
 }
 
-static int curve_pen_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus curve_pen_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   ViewContext vc = ED_view3d_viewcontext_init(C, depsgraph);
@@ -1755,7 +1750,7 @@ static int curve_pen_invoke(bContext *C, wmOperator *op, const wmEvent *event)
   Nurb *nu = nullptr;
 
   CurvePenData *cpd;
-  op->customdata = cpd = static_cast<CurvePenData *>(MEM_callocN(sizeof(CurvePenData), __func__));
+  op->customdata = cpd = MEM_callocN<CurvePenData>(__func__);
 
   /* Distance threshold for mouse clicks to affect the spline or its points */
   const float mval_fl[2] = {float(event->mval[0]), float(event->mval[1])};
@@ -1839,7 +1834,7 @@ void CURVE_OT_pen(wmOperatorType *ot)
   ot->idname = "CURVE_OT_pen";
   ot->description = "Construct and edit splines";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = curve_pen_invoke;
   ot->modal = curve_pen_modal;
   ot->poll = ED_operator_editcurve;
