@@ -104,16 +104,16 @@ static void uilist_draw_item_default(uiList *ui_list,
   /* Simplest one! */
   switch (ui_list->layout_type) {
     case UILST_LAYOUT_GRID:
-      uiItemL(layout, "", icon);
+      layout->label("", icon);
       break;
     case UILST_LAYOUT_DEFAULT:
     case UILST_LAYOUT_COMPACT:
     default:
       if (nameprop) {
-        uiItemFullR(layout, itemptr, nameprop, RNA_NO_INDEX, 0, UI_ITEM_R_NO_BG, "", icon);
+        layout->prop(itemptr, nameprop, RNA_NO_INDEX, 0, UI_ITEM_R_NO_BG, "", icon);
       }
       else {
-        uiItemL(layout, "", icon);
+        layout->label("", icon);
       }
       break;
   }
@@ -123,31 +123,26 @@ static void uilist_draw_filter_default(uiList *ui_list, const bContext * /*C*/,
 {
   PointerRNA listptr = RNA_pointer_create_discrete(nullptr, &RNA_UIList, ui_list);
 
-  uiLayout *row = uiLayoutRow(layout, false);
+  uiLayout *row = &layout->row(false);
 
-  uiLayout *subrow = uiLayoutRow(row, true);
-  uiItemR(subrow, &listptr, "filter_name", UI_ITEM_NONE, "", ICON_NONE);
-  uiItemR(subrow,
-          &listptr,
-          "use_filter_invert",
-          UI_ITEM_R_TOGGLE | UI_ITEM_R_ICON_ONLY,
-          "",
-          ICON_ARROW_LEFTRIGHT);
+  uiLayout *subrow = &row->row(true);
+  subrow->prop(&listptr, "filter_name", UI_ITEM_NONE, "", ICON_NONE);
+  subrow->prop(&listptr,
+               "use_filter_invert",
+               UI_ITEM_R_TOGGLE | UI_ITEM_R_ICON_ONLY,
+               "",
+               ICON_ARROW_LEFTRIGHT);
 
   if ((ui_list->filter_sort_flag & UILST_FLT_SORT_LOCK) == 0) {
-    subrow = uiLayoutRow(row, true);
-    uiItemR(subrow,
-            &listptr,
-            "use_filter_sort_alpha",
-            UI_ITEM_R_TOGGLE | UI_ITEM_R_ICON_ONLY,
-            "",
-            ICON_NONE);
-    uiItemR(subrow,
-            &listptr,
-            "use_filter_sort_reverse",
-            UI_ITEM_R_TOGGLE | UI_ITEM_R_ICON_ONLY,
-            "",
-            (ui_list->filter_sort_flag & UILST_FLT_SORT_REVERSE) ? ICON_SORT_DESC : ICON_SORT_ASC);
+    subrow = &row->row(true);
+    subrow->prop(
+        &listptr, "use_filter_sort_alpha", UI_ITEM_R_TOGGLE | UI_ITEM_R_ICON_ONLY, "", ICON_NONE);
+    subrow->prop(&listptr,
+                 "use_filter_sort_reverse",
+                 UI_ITEM_R_TOGGLE | UI_ITEM_R_ICON_ONLY,
+                 "",
+                 (ui_list->filter_sort_flag & UILST_FLT_SORT_REVERSE) ? ICON_SORT_DESC :
+                                                                        ICON_SORT_ASC);
   }
 }
 
@@ -163,8 +158,7 @@ uiListNameFilter::uiListNameFilter(uiList &list)
       filter_ = storage_.filter_buff;
     }
     else {
-      filter_ = storage_.filter_dyn = static_cast<char *>(
-          MEM_mallocN((slen + 3) * sizeof(char), "filter_dyn"));
+      filter_ = storage_.filter_dyn = MEM_malloc_arrayN<char>((slen + 3), "filter_dyn");
     }
     BLI_strncpy_ensure_pad(filter_, filter_raw, '*', slen + 3);
   }
@@ -228,12 +222,11 @@ void UI_list_filter_and_sort_items(uiList *ui_list,
     int order_idx = 0, i = 0;
 
     if (order_by_name) {
-      names = static_cast<StringCmp *>(MEM_callocN(sizeof(StringCmp) * len, "StringCmp"));
+      names = MEM_calloc_arrayN<StringCmp>(len, "StringCmp");
     }
 
     if (item_filter_fn) {
-      dyn_data->items_filter_flags = static_cast<int *>(
-          MEM_callocN(sizeof(int) * len, "items_filter_flags"));
+      dyn_data->items_filter_flags = MEM_calloc_arrayN<int>(len, "items_filter_flags");
       dyn_data->items_shown = 0;
     }
 
@@ -257,14 +250,15 @@ void UI_list_filter_and_sort_items(uiList *ui_list,
           dyn_data->items_filter_flags[i] = UILST_FLT_ITEM_NEVER_SHOW;
         }
         else if (filter_result == UI_LIST_ITEM_FILTER_MATCHES) {
-          dyn_data->items_filter_flags[i] = UILST_FLT_ITEM;
           if (!filter_exclude) {
+            dyn_data->items_filter_flags[i] = UILST_FLT_ITEM;
             dyn_data->items_shown++;
             do_order = order_by_name;
           }
           // printf("%s: '%s' matches '%s'\n", __func__, name, filter);
         }
         else if (filter_exclude) {
+          dyn_data->items_filter_flags[i] = UILST_FLT_ITEM;
           dyn_data->items_shown++;
           do_order = order_by_name;
         }
@@ -295,8 +289,7 @@ void UI_list_filter_and_sort_items(uiList *ui_list,
        */
       qsort(names, order_idx, sizeof(StringCmp), cmpstringp);
 
-      dyn_data->items_filter_neworder = static_cast<int *>(
-          MEM_mallocN(sizeof(int) * order_idx, "items_filter_neworder"));
+      dyn_data->items_filter_neworder = MEM_malloc_arrayN<int>(order_idx, "items_filter_neworder");
       for (new_idx = 0; new_idx < order_idx; new_idx++) {
         dyn_data->items_filter_neworder[names[new_idx].org_idx] = new_idx;
       }
@@ -321,8 +314,7 @@ bool UI_list_item_index_is_filtered_visible(const uiList *ui_list, const int ite
     return false;
   }
 
-  const int filter_exclude = ui_list->filter_flag & UILST_FLT_EXCLUDE;
-  return (dyn_data->items_filter_flags[item_idx] & UILST_FLT_ITEM) ^ filter_exclude;
+  return (dyn_data->items_filter_flags[item_idx] & UILST_FLT_ITEM);
 }
 
 /**
@@ -639,11 +631,11 @@ static void *uilist_item_use_dynamic_tooltip(PointerRNA *itemptr, const char *pr
   return nullptr;
 }
 
-static std::string uilist_item_tooltip_func(bContext * /*C*/, void *argN, const char *tip)
+static std::string uilist_item_tooltip_func(bContext * /*C*/, void *argN, const StringRef tip)
 {
   char *dyn_tooltip = static_cast<char *>(argN);
   std::string tooltip_string = dyn_tooltip;
-  if (tip && tip[0]) {
+  if (!tip.is_empty()) {
     tooltip_string += '\n';
     tooltip_string += tip;
   }
@@ -675,7 +667,7 @@ static uiList *ui_list_ensure(const bContext *C,
       BLI_findstring(&region->ui_lists, full_list_id, offsetof(uiList, list_id)));
 
   if (!ui_list) {
-    ui_list = static_cast<uiList *>(MEM_callocN(sizeof(uiList), "uiList"));
+    ui_list = MEM_callocN<uiList>("uiList");
     STRNCPY(ui_list->list_id, full_list_id);
     BLI_addtail(&region->ui_lists, ui_list);
     ui_list->list_grip = -UI_LIST_AUTO_SIZE_THRESHOLD; /* Force auto size by default. */
@@ -688,8 +680,7 @@ static uiList *ui_list_ensure(const bContext *C,
   }
 
   if (!ui_list->dyn_data) {
-    ui_list->dyn_data = static_cast<uiListDyn *>(
-        MEM_callocN(sizeof(uiListDyn), "uiList.dyn_data"));
+    ui_list->dyn_data = MEM_callocN<uiListDyn>("uiList.dyn_data");
   }
   uiListDyn *dyn_data = ui_list->dyn_data;
   /* Note that this isn't a `uiListType` callback, it's stored in the runtime list data. Otherwise
@@ -736,10 +727,10 @@ static void ui_template_list_layout_draw(const bContext *C,
   switch (ui_list->layout_type) {
     case UILST_LAYOUT_DEFAULT: {
       /* layout */
-      box = uiLayoutListBox(layout, ui_list, &input_data->active_dataptr, input_data->activeprop);
-      glob = uiLayoutColumn(box, true);
-      row = uiLayoutRow(glob, false);
-      col = uiLayoutColumn(row, true);
+      box = &layout->list_box(ui_list, &input_data->active_dataptr, input_data->activeprop);
+      glob = &box->column(true);
+      row = &glob->row(false);
+      col = &row->column(true);
 
       TemplateListLayoutDrawData adjusted_layout_data = *layout_data;
       adjusted_layout_data.columns = 1;
@@ -759,12 +750,12 @@ static void ui_template_list_layout_draw(const bContext *C,
           const int flt_flag = items->item_vec[i].flt_flag;
           uiBlock *subblock = uiLayoutGetBlock(col);
 
-          overlap = uiLayoutOverlap(col);
+          overlap = &col->overlap();
 
           UI_block_flag_enable(subblock, UI_BLOCK_LIST_ITEM);
 
           /* list item behind label & other buttons */
-          uiLayoutRow(overlap, false);
+          overlap->row(false);
 
           but = uiDefButR_prop(subblock,
                                UI_BTYPE_LISTROW,
@@ -789,11 +780,11 @@ static void ui_template_list_layout_draw(const bContext *C,
             UI_but_func_tooltip_set(but, uilist_item_tooltip_func, dyntip_data, MEM_freeN);
           }
 
-          uiLayout *item_row = uiLayoutRow(overlap, true);
+          uiLayout *item_row = &overlap->row(true);
 
           uiLayoutListItemAddPadding(item_row);
 
-          sub = uiLayoutRow(item_row, false);
+          sub = &item_row->row(false);
           icon = UI_icon_from_rnaptr(C, itemptr, rnaicon, false);
           if (icon == ICON_DOT) {
             icon = ICON_NONE;
@@ -825,12 +816,12 @@ static void ui_template_list_layout_draw(const bContext *C,
 
       /* add dummy buttons to fill space */
       for (; i < visual_info.start_idx + visual_info.visual_items; i++) {
-        uiItemL(col, "", ICON_NONE);
+        col->label("", ICON_NONE);
       }
 
       /* Add scroll-bar. */
       if (items->item_vec.size() > visual_info.visual_items) {
-        uiLayoutColumn(row, false);
+        row->column(false);
         but = uiDefButI(block,
                         UI_BTYPE_SCROLL,
                         0,
@@ -849,7 +840,7 @@ static void ui_template_list_layout_draw(const bContext *C,
       break;
     }
     case UILST_LAYOUT_COMPACT:
-      row = uiLayoutRow(layout, true);
+      row = &layout->row(true);
 
       if ((input_data->dataptr.data && input_data->prop) && (dyn_data->items_shown > 0) &&
           (items->active_item_idx >= 0) && (items->active_item_idx < dyn_data->items_shown))
@@ -874,7 +865,7 @@ static void ui_template_list_layout_draw(const bContext *C,
       }
       /* if list is empty, add in dummy button */
       else {
-        uiItemL(row, "", ICON_NONE);
+        row->label("", ICON_NONE);
       }
 
       /* next/prev button */
@@ -899,10 +890,10 @@ static void ui_template_list_layout_draw(const bContext *C,
       }
       break;
     case UILST_LAYOUT_GRID: {
-      box = uiLayoutListBox(layout, ui_list, &input_data->active_dataptr, input_data->activeprop);
-      glob = uiLayoutColumn(box, true);
-      row = uiLayoutRow(glob, false);
-      col = uiLayoutColumn(row, true);
+      box = &layout->list_box(ui_list, &input_data->active_dataptr, input_data->activeprop);
+      glob = &box->column(true);
+      row = &glob->row(false);
+      col = &row->column(true);
       subrow = nullptr; /* Quite gcc warning! */
 
       uilist_prepare(ui_list, items, layout_data, &visual_info);
@@ -917,16 +908,16 @@ static void ui_template_list_layout_draw(const bContext *C,
 
           /* create button */
           if (!(i % layout_data->columns)) {
-            subrow = uiLayoutRow(col, false);
+            subrow = &col->row(false);
           }
 
           uiBlock *subblock = uiLayoutGetBlock(subrow);
-          overlap = uiLayoutOverlap(subrow);
+          overlap = &subrow->overlap();
 
           UI_block_flag_enable(subblock, UI_BLOCK_LIST_ITEM);
 
           /* list item behind label & other buttons */
-          uiLayoutRow(overlap, false);
+          overlap->row(false);
 
           but = uiDefButR_prop(subblock,
                                UI_BTYPE_LISTROW,
@@ -941,10 +932,10 @@ static void ui_template_list_layout_draw(const bContext *C,
                                0,
                                0,
                                org_i,
-                               nullptr);
+                               std::nullopt);
           UI_but_drawflag_enable(but, UI_BUT_NO_TOOLTIP);
 
-          sub = uiLayoutRow(overlap, false);
+          sub = &overlap->row(false);
 
           icon = UI_icon_from_rnaptr(C, itemptr, rnaicon, false);
           layout_data->draw_item(ui_list,
@@ -970,14 +961,14 @@ static void ui_template_list_layout_draw(const bContext *C,
       /* add dummy buttons to fill space */
       for (; i < visual_info.start_idx + visual_info.visual_items; i++) {
         if (!(i % layout_data->columns)) {
-          subrow = uiLayoutRow(col, false);
+          subrow = &col->row(false);
         }
-        uiItemL(subrow, "", ICON_NONE);
+        subrow->label("", ICON_NONE);
       }
 
       /* Add scroll-bar. */
       if (items->item_vec.size() > visual_info.visual_items) {
-        /* col = */ uiLayoutColumn(row, false);
+        /* col = */ row->column(false);
         but = uiDefButI(block,
                         UI_BTYPE_SCROLL,
                         0,
@@ -996,11 +987,11 @@ static void ui_template_list_layout_draw(const bContext *C,
       break;
     }
     case UILST_LAYOUT_BIG_PREVIEW_GRID:
-      box = uiLayoutListBox(layout, ui_list, &input_data->active_dataptr, input_data->activeprop);
+      box = &layout->list_box(ui_list, &input_data->active_dataptr, input_data->activeprop);
       /* For grip button. */
-      glob = uiLayoutColumn(box, true);
+      glob = &box->column(true);
       /* For scroll-bar. */
-      row = uiLayoutRow(glob, false);
+      row = &glob->row(false);
 
       const bool show_names = (flags & UI_TEMPLATE_LIST_NO_NAMES) == 0;
 
@@ -1009,7 +1000,7 @@ static void ui_template_list_layout_draw(const bContext *C,
 
       const int cols_per_row = std::max(int((uiLayoutGetWidth(box) - V2D_SCROLL_WIDTH) / size_x),
                                         1);
-      uiLayout *grid = uiLayoutGridFlow(row, true, cols_per_row, true, true, true);
+      uiLayout *grid = &row->grid_flow(true, cols_per_row, true, true, true);
 
       TemplateListLayoutDrawData adjusted_layout_data = *layout_data;
       adjusted_layout_data.columns = cols_per_row;
@@ -1022,8 +1013,8 @@ static void ui_template_list_layout_draw(const bContext *C,
           const int org_i = items->item_vec[i].org_idx;
           const int flt_flag = items->item_vec[i].flt_flag;
 
-          overlap = uiLayoutOverlap(grid);
-          col = uiLayoutColumn(overlap, false);
+          overlap = &grid->overlap();
+          col = &overlap->column(false);
 
           uiBlock *subblock = uiLayoutGetBlock(col);
           UI_block_flag_enable(subblock, UI_BLOCK_LIST_ITEM);
@@ -1041,10 +1032,10 @@ static void ui_template_list_layout_draw(const bContext *C,
                                0,
                                0,
                                org_i,
-                               nullptr);
+                               std::nullopt);
           UI_but_drawflag_enable(but, UI_BUT_NO_TOOLTIP);
 
-          col = uiLayoutColumn(overlap, false);
+          col = &overlap->column(false);
 
           icon = UI_icon_from_rnaptr(C, itemptr, rnaicon, false);
           layout_data->draw_item(ui_list,
@@ -1072,7 +1063,7 @@ static void ui_template_list_layout_draw(const bContext *C,
       }
 
       if (items->item_vec.size() > visual_info.visual_items) {
-        /* col = */ uiLayoutColumn(row, false);
+        /* col = */ row->column(false);
         but = uiDefButI(block,
                         UI_BTYPE_SCROLL,
                         0,
@@ -1111,9 +1102,9 @@ static void ui_template_list_layout_draw(const bContext *C,
     dyn_data->resize = dyn_data->resize_prev +
                        (dyn_data->visual_height - ui_list->list_grip) * UI_UNIT_Y;
 
-    row = uiLayoutRow(glob, true);
+    row = &glob->row(true);
     uiBlock *subblock = uiLayoutGetBlock(row);
-    UI_block_emboss_set(subblock, UI_EMBOSS_NONE);
+    UI_block_emboss_set(subblock, blender::ui::EmbossType::None);
 
     if (ui_list->filter_flag & UILST_FLT_SHOW) {
       but = uiDefIconButBitI(subblock,
@@ -1147,9 +1138,9 @@ static void ui_template_list_layout_draw(const bContext *C,
         UI_but_func_set(but, [ui_list](bContext &C) { uilist_resize_update(&C, ui_list); });
       }
 
-      UI_block_emboss_set(subblock, UI_EMBOSS);
+      UI_block_emboss_set(subblock, blender::ui::EmbossType::Emboss);
 
-      col = uiLayoutColumn(glob, false);
+      col = &glob->column(false);
       subblock = uiLayoutGetBlock(col);
       uiDefBut(subblock,
                UI_BTYPE_SEPR,
@@ -1198,7 +1189,7 @@ static void ui_template_list_layout_draw(const bContext *C,
         UI_but_func_set(but, [ui_list](bContext &C) { uilist_resize_update(&C, ui_list); });
       }
 
-      UI_block_emboss_set(subblock, UI_EMBOSS);
+      UI_block_emboss_set(subblock, blender::ui::EmbossType::Emboss);
     }
   }
 }
