@@ -12,10 +12,6 @@
 
 #include "../../../view_map/Silhouette.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////////////////
 
 extern PyTypeObject FEdgeSmooth_Type;
@@ -34,7 +30,3 @@ typedef struct {
 void FEdgeSmooth_mathutils_register_callback();
 
 ///////////////////////////////////////////////////////////////////////////////////////////
-
-#ifdef __cplusplus
-}
-#endif
