@@ -6,10 +6,12 @@
  * \ingroup gpu
  */
 
+#include "BKE_material.hh"
+
+#include "GPU_batch.hh"
 #include "GPU_init_exit.hh" /* interface */
-#include "GPU_batch.hh"
+#include "GPU_pass.hh"
 
-#include "intern/gpu_codegen.hh"
 #include "intern/gpu_private.hh"
 #include "intern/gpu_shader_create_info_private.hh"
 #include "intern/gpu_shader_dependency_private.hh"
@@ -29,10 +31,12 @@ void GPU_init()
 
   initialized = true;
 
+  gpu_backend_init_resources();
+
   gpu_shader_dependency_init();
   gpu_shader_create_info_init();
 
-  gpu_codegen_init();
+  GPU_pass_cache_init();
 
   gpu_batch_init();
 }
@@ -41,13 +45,16 @@ void GPU_exit()
 {
   gpu_batch_exit();
 
-  gpu_codegen_exit();
+  GPU_pass_cache_free();
+
+  BKE_material_defaults_free_gpu();
+  GPU_shader_free_builtin_shaders();
+
+  gpu_backend_delete_resources();
 
   gpu_shader_dependency_exit();
   gpu_shader_create_info_exit();
 
-  gpu_backend_delete_resources();
-
   initialized = false;
 }
 
