@@ -10,10 +10,12 @@ namespace blender::nodes::node_fn_transform_direction_cc {
 
 static void node_declare(NodeDeclarationBuilder &b)
 {
+  b.use_custom_socket_order();
+  b.allow_any_socket_order();
   b.is_function_node();
   b.add_input<decl::Vector>("Direction").subtype(PROP_XYZ);
+  b.add_output<decl::Vector>("Direction").subtype(PROP_XYZ).align_with_previous();
   b.add_input<decl::Matrix>("Transform");
-  b.add_output<decl::Vector>("Direction").subtype(PROP_XYZ);
 }
 
 static void node_build_multi_function(NodeMultiFunctionBuilder &builder)
@@ -34,7 +36,7 @@ static void node_register()
   ntype.nclass = NODE_CLASS_CONVERTER;
   ntype.declare = node_declare;
   ntype.build_multi_function = node_build_multi_function;
-  blender::bke::node_register_type(&ntype);
+  blender::bke::node_register_type(ntype);
 }
 NOD_REGISTER_NODE(node_register)
 
