@@ -155,9 +155,9 @@ PyDoc_STRVAR(
     "\n"
     "   :arg callback:\n"
     "      A function that will be called when the cursor is drawn.\n"
-    "      It gets the specified arguments as input with the mouse position (tuple) as last "
-    "argument.\n"
-    "   :type callback: Callable[[Any, ..., tuple[int, int]], Any]\n"
+    "      It gets the specified arguments as input with the mouse position "
+    "(``tuple[int, int]``) as last argument.\n"
+    "   :type callback: Callable[..., Any]\n"
     "   :arg args: Arguments that will be passed to the callback.\n"
     "   :type args: tuple[Any, ...]\n"
     "   :arg space_type: The space type the callback draws in; for example ``VIEW_3D``. "
@@ -229,7 +229,7 @@ PyDoc_STRVAR(
     "   :arg callback:\n"
     "      A function that will be called when the region is drawn.\n"
     "      It gets the specified arguments as input, it's return value is ignored.\n"
-    "   :type callback: Callable[[Any, ...], Any]\n"
+    "   :type callback: Callable[..., Any]\n"
     "   :arg args: Arguments that will be passed to the callback.\n"
     "   :type args: tuple[Any, ...]\n"
     "   :arg region_type: The region type the callback draws in; usually ``WINDOW``. "
