@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: 2021-2023 Blender Foundation
+# SPDX-FileCopyrightText: 2021-2023 Blender Authors
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
@@ -14,7 +14,25 @@ import bpy
 args = None
 
 
-type_info = {
+base_idname = {
+    "VALUE": "NodeSocketFloat",
+    "INT": "NodeSocketInt",
+    "BOOLEAN": "NodeSocketBool",
+    "ROTATION": "NodeSocketRotation",
+    "VECTOR": "NodeSocketVector",
+    "RGBA": "NodeSocketColor",
+    "STRING": "NodeSocketString",
+    "SHADER": "NodeSocketShader",
+    "OBJECT": "NodeSocketObject",
+    "IMAGE": "NodeSocketImage",
+    "GEOMETRY": "NodeSocketGeometry",
+    "COLLECTION": "NodeSocketCollection",
+    "TEXTURE": "NodeSocketTexture",
+    "MATERIAL": "NodeSocketMaterial",
+}
+
+
+subtype_idname = {
     ("VALUE", "NONE"): "NodeSocketFloat",
     ("VALUE", "UNSIGNED"): "NodeSocketFloatUnsigned",
     ("VALUE", "PERCENTAGE"): "NodeSocketFloatPercentage",
@@ -30,6 +48,8 @@ type_info = {
     ("BOOLEAN", "NONE"): "NodeSocketBool",
     ("ROTATION", "NONE"): "NodeSocketRotation",
     ("VECTOR", "NONE"): "NodeSocketVector",
+    ("VECTOR", "FACTOR"): "NodeSocketVectorFactor",
+    ("VECTOR", "PERCENTAGE"): "NodeSocketVectorPercentage",
     ("VECTOR", "TRANSLATION"): "NodeSocketVectorTranslation",
     ("VECTOR", "DIRECTION"): "NodeSocketVectorDirection",
     ("VECTOR", "VELOCITY"): "NodeSocketVectorVelocity",
@@ -38,6 +58,7 @@ type_info = {
     ("VECTOR", "XYZ"): "NodeSocketVectorXYZ",
     ("RGBA", "NONE"): "NodeSocketColor",
     ("STRING", "NONE"): "NodeSocketString",
+    ("STRING", "FILEPATH"): "NodeSocketStringFilePath",
     ("SHADER", "NONE"): "NodeSocketShader",
     ("OBJECT", "NONE"): "NodeSocketObject",
     ("IMAGE", "NONE"): "NodeSocketImage",
@@ -63,8 +84,12 @@ class SocketSpec():
     external_links: int = 1
 
     @property
-    def idname(self):
-        return type_info[(self.type, self.subtype)]
+    def base_idname(self):
+        return base_idname[self.type]
+
+    @property
+    def subtype_idname(self):
+        return subtype_idname[(self.type, self.subtype)]
 
 
 class AbstractNodeGroupInterfaceTest(unittest.TestCase):
@@ -95,7 +120,7 @@ class AbstractNodeGroupInterfaceTest(unittest.TestCase):
 
         # Examine the interface item.
         self.assertEqual(item.name, spec.name)
-        self.assertEqual(item.bl_socket_idname, spec.idname)
+        self.assertEqual(item.bl_socket_idname, spec.base_idname)
         self.assertEqual(item.identifier, spec.identifier)
 
         # Types that have subtypes.
@@ -134,7 +159,7 @@ class AbstractNodeGroupInterfaceTest(unittest.TestCase):
             socket = next(s for s in node.inputs if s.identifier == spec.identifier)
             self.assertIsNotNone(socket, f"Could not find socket for group input identifier {spec.identifier}")
             self.assertEqual(socket.name, spec.name)
-            self.assertEqual(socket.bl_idname, spec.idname)
+            self.assertEqual(socket.bl_idname, spec.subtype_idname)
             self.assertEqual(socket.type, spec.type)
             self.assertEqual(socket.hide_value, spec.hide_value)
             if test_links:
@@ -147,7 +172,7 @@ class AbstractNodeGroupInterfaceTest(unittest.TestCase):
             self.assertIsNotNone(
                 socket, f"Could not find group input socket for group input identifier {spec.identifier}")
             self.assertEqual(socket.name, spec.name)
-            self.assertEqual(socket.bl_idname, spec.idname)
+            self.assertEqual(socket.bl_idname, spec.subtype_idname)
             self.assertEqual(socket.type, spec.type)
             self.assertEqual(socket.hide_value, spec.hide_value)
             if test_links:
@@ -158,7 +183,7 @@ class AbstractNodeGroupInterfaceTest(unittest.TestCase):
             socket = next(s for s in node.outputs if s.identifier == spec.identifier)
             self.assertIsNotNone(socket, f"Could not find socket for group output identifier {spec.identifier}")
             self.assertEqual(socket.name, spec.name)
-            self.assertEqual(socket.bl_idname, spec.idname)
+            self.assertEqual(socket.bl_idname, spec.subtype_idname)
             self.assertEqual(socket.type, spec.type)
             self.assertEqual(socket.hide_value, spec.hide_value)
             if test_links:
@@ -171,7 +196,7 @@ class AbstractNodeGroupInterfaceTest(unittest.TestCase):
             self.assertIsNotNone(
                 socket, f"Could not find group output socket for group output identifier {spec.identifier}")
             self.assertEqual(socket.name, spec.name)
-            self.assertEqual(socket.bl_idname, spec.idname)
+            self.assertEqual(socket.bl_idname, spec.subtype_idname)
             self.assertEqual(socket.type, spec.type)
             self.assertEqual(socket.hide_value, spec.hide_value)
             if test_links:
@@ -201,11 +226,11 @@ class NodeGroupVersioning36Test(AbstractNodeGroupInterfaceTest):
         # autopep8: off
         self.compare_group_to_specs(group, node, [
             SocketSpec("Output Float", "Output_9", "VALUE", hide_value=True, default_value=3.0, min_value=1.0, max_value=1.0),
-            SocketSpec("Output Vector", "Output_10", "VECTOR", subtype="EULER", default_value=( 10, 20, 30), min_value=-10.0, max_value=10.0),
+            SocketSpec("Output Vector", "Output_10", "VECTOR", subtype="EULER", default_value=(10, 20, 30), min_value=-10.0, max_value=10.0),
             SocketSpec("Output Color", "Output_11", "RGBA", default_value=(0, 1, 1, 1)),
 
             SocketSpec("Input Float", "Input_6", "VALUE", subtype="ANGLE", default_value=-20.0, min_value=5.0, max_value=6.0),
-            SocketSpec("Input Vector", "Input_7", "VECTOR", hide_value=True, default_value=( 2, 4, 6), min_value=-4.0, max_value=100.0),
+            SocketSpec("Input Vector", "Input_7", "VECTOR", hide_value=True, default_value=(2, 4, 6), min_value=-4.0, max_value=100.0),
             SocketSpec("Input Color", "Input_8", "RGBA", default_value=(0.5, 0.4, 0.3, 0.2)),
         ])
         # autopep8: on
@@ -222,12 +247,12 @@ class NodeGroupVersioning36Test(AbstractNodeGroupInterfaceTest):
         # autopep8: off
         self.compare_group_to_specs(group, node, [
             SocketSpec("Output Float", "Output_30", "VALUE", hide_value=True, default_value=3.0, min_value=1.0, max_value=1.0),
-            SocketSpec("Output Vector", "Output_31", "VECTOR", subtype="EULER", default_value=( 10, 20, 30), min_value=-10.0, max_value=10.0),
+            SocketSpec("Output Vector", "Output_31", "VECTOR", subtype="EULER", default_value=(10, 20, 30), min_value=-10.0, max_value=10.0),
             SocketSpec("Output Color", "Output_32", "RGBA", default_value=(0, 1, 1, 1)),
             SocketSpec("Output Shader", "Output_33", "SHADER"),
 
             SocketSpec("Input Float", "Input_26", "VALUE", subtype="ANGLE", default_value=-20.0, min_value=5.0, max_value=6.0),
-            SocketSpec("Input Vector", "Input_27", "VECTOR", hide_value=True, default_value=( 2, 4, 6), min_value=-4.0, max_value=100.0),
+            SocketSpec("Input Vector", "Input_27", "VECTOR", hide_value=True, default_value=(2, 4, 6), min_value=-4.0, max_value=100.0),
             SocketSpec("Input Color", "Input_28", "RGBA", default_value=(0.5, 0.4, 0.3, 0.2)),
             SocketSpec("Input Shader", "Input_29", "SHADER"),
         ])
@@ -245,7 +270,7 @@ class NodeGroupVersioning36Test(AbstractNodeGroupInterfaceTest):
         # autopep8: off
         self.compare_group_to_specs(group, node, [
             SocketSpec("Output Float", "Output_7", "VALUE", hide_value=True, default_value=3.0, min_value=1.0, max_value=1.0),
-            SocketSpec("Output Vector", "Output_8", "VECTOR", subtype="EULER", default_value=( 10, 20, 30), min_value=-10.0, max_value=10.0),
+            SocketSpec("Output Vector", "Output_8", "VECTOR", subtype="EULER", default_value=(10, 20, 30), min_value=-10.0, max_value=10.0),
             SocketSpec("Output Color", "Output_9", "RGBA", default_value=(0, 1, 1, 1)),
             SocketSpec("Output String", "Output_19", "STRING", default_value=""),
             SocketSpec("Output Bool", "Output_20", "BOOLEAN", default_value=False),
@@ -258,7 +283,7 @@ class NodeGroupVersioning36Test(AbstractNodeGroupInterfaceTest):
             SocketSpec("Output Image", "Output_27", "IMAGE", default_value=bpy.data.images['TestImage']),
 
             SocketSpec("Input Float", "Input_4", "VALUE", subtype="ANGLE", default_value=-20.0, min_value=5.0, max_value=6.0),
-            SocketSpec("Input Vector", "Input_5", "VECTOR", hide_value=True, default_value=( 2, 4, 6), min_value=-4.0, max_value=100.0),
+            SocketSpec("Input Vector", "Input_5", "VECTOR", hide_value=True, default_value=(2, 4, 6), min_value=-4.0, max_value=100.0),
             SocketSpec("Input Color", "Input_6", "RGBA", default_value=(0.5, 0.4, 0.3, 0.2)),
             SocketSpec("Input String", "Input_10", "STRING", default_value="hello world!"),
             SocketSpec("Input Bool", "Input_11", "BOOLEAN", default_value=True, hide_in_modifier=True),
